@@ -152,17 +152,21 @@ class EvalObject(object):
 def PredictionEval(
     PredictionObject,
     actual,
-    series_weights: dict = {},
-    df_train=np.nan,
+    series_weights: dict,
+    df_train=None,
     per_timestamp_errors: bool = False,
     dist_n: int = None,
 ):
     """Evalute prediction against test actual.
 
+    This fails with pd.NA values supplied.
+
     Args:
         PredictionObject (autots.PredictionObject): Prediction from AutoTS model object
         actual (pd.DataFrame): dataframe of actual values of (forecast length * n series)
         series_weights (dict): key = column/series_id, value = weight
+        df_train (pd.DataFrame): historical values of series, used for setting scaler for SPL
+            if None, actuals are used instead. Suboptimal.
         per_timestamp (bool): whether to calculate and return per timestamp direction errors
         dist_n (int): if not None, calculates two part rmse on head(n) and tail(remainder) of forecast.
     """
@@ -170,7 +174,14 @@ def PredictionEval(
     F = np.array(PredictionObject.forecast)
     lower_forecast = np.array(PredictionObject.lower_forecast)
     upper_forecast = np.array(PredictionObject.upper_forecast)
+    if df_train is None:
+        df_train = actual
     df_train = np.array(df_train)
+    # make sure the series_weights are passed correctly to metrics
+    if len(series_weights) != F.shape[1]:
+        series_weights = {
+            col: series_weights[col] for col in PredictionObject.forecast.columns
+        }
 
     errors = EvalObject(model_name=PredictionObject.model_name)
 
